-- vim: ts=2 sw=2 ai et
--
-- Copyright(c) 2008 Johny Mattsson
--      a.k.a.
-- Anyia of HordeYakka (Jubei'Thos)
--
-- Permission to copy and re-use as seen fit.
-- Credit would be nice.
--

local NAME = "TwinCam"
local ADDON_NAME = "TwinCam_Overlay"
TwinCam = LibStub ("AceAddon-3.0"):NewAddon ("TwinCam", "AceConsole-3.0", "AceEvent-3.0")

local options = {
  type = 'group',
  handler = TwinCam,
  childGroups = 'tab',
  args = {
    warnings = {
      type = 'group',
      name = "Warnings",
      order = 1,
      args = {
        conflagration = {
          type = 'group',
          name = "Conflagration",
          desc = "Conflagration warning settings",
          order = 1,
          args = {
            conflagSound = {
              type = 'toggle',
              order = 1,
              name = "Play sound",
              desc = "Play a warning sound when a conflagration is being cast at you",
              get = "GetBool",
              set = "SetBool",
            },
            conflagVisible = {
              type = 'toggle',
              order = 2,
              name = "Flash screen",
              desc = "Flash a skull overlay across the screen when a conflagration is being cast at you",
              get = "GetBool",
              set = "SetBool",
            },
            conflagColor = {
              type = 'color',
              hasAlpha = false,
              order = 3,
              name = "Skull color",
              desc = "Color/intensity of the skull overlay",
              get = "GetColor",
              set = "SetColor",
            },
            conflagPreview = {
              type = 'execute',
              name = "Preview",
              desc = "Preview Conflagration warning",
              func = "OnConflag",
            },
          }
        },
        shadowfury = {
          type = 'group',
          name = "Shadowfury",
          desc = "Shadowfury stun warning settings",
          order = 2,
          args = {
            shadowfuryVisible = {
              type = 'toggle',
              name = "Tint screen",
              desc = "Tint the screen when you are stunned by Shadowfury",
              order = 1,
              get = "GetBool",
              set = "SetBool",
            },
            shadowfuryColor = {
              type = 'color',
              hasAlpha = false,
              name = "Tint color",
              desc = "Color to tint the screen during Shadowfury stun",
              order = 2,
              get = "GetColor",
              set = "SetColor",
            },
            shadowfuryPreview = {
              type = 'execute',
              name = "Preview",
              desc = "Preview Shadowfury stun warning",
              func = "OnShadowfury",
            },
            shadowfuryPreviewOff = {
              type = 'execute',
              name = "Preview Off",
              desc = "Stop previewing the Shadowfury stun warning",
              func = "StopFlashShadowfury",
            },
          },
        },
      },
    },

    about = {
      type = 'group',
      name = "About",
      desc = "About " .. ADDON_NAME,
      order = -1,
      args = {
        __ = {
          type = 'header',
          name = ADDON_NAME .. " " .. GetAddOnMetadata (ADDON_NAME, "Version"),
          order = 1,
        },
        _1 = {
          type = 'description',
          name = GetAddOnMetadata (ADDON_NAME, "Notes"),
          order = 10,
        },
        _2 = {
          type = 'description',
          name = "By " .. GetAddOnMetadata(ADDON_NAME, "Author"),
          order = 20,
        },
        _3 = {
          type = 'description',
          name = "",
          order = 30,
        },
        _4 = {
          type = 'description',
          name = "Copyright(C) 2008 Johny Mattsson",
          order = 40,
        },
        _5 = {
          type = 'description',
          name = "",
          order = 50,
        },
        _6 = {
          type = 'description',
          name = "Build: " .. GetAddOnMetadata (ADDON_NAME, "X-Build"):match("%d+"),
          order = 60,
        },
      },
    }
  }
}
local defaults = {
  profile = {
    conflagSound = true,
    conflagVisible = true,
    conflagColor = { r = 0.6, g = 0.6, b = 0.6, a = 1 },
    shadowfuryVisible = true,
    shadowfuryColor = { r = 1, g = 0, b = 0, a = 1 },
  }
}

-- "Constants"
local GWA = "Grand Warlock Alythess"
local LS = "Lady Sacrolash"
local RunAwaySound = "Sound/Creature/HoodWolf/HoodWolfTransformPlayer01.wav"
local LamentSound = "Sound/Music/GlueScreenMusic/BCCredits_Lament_of_the_Highborne.mp3"
-- We actually snarf the texture straight from the TargetRaidTargetIcon frame,
-- but in case we ever need to remember where it is, this is it
local RaidTargetsTextures = "Interface/TargetingFrame/UI-RaidTargetingIcons"

-- Config manipulators
function TwinCam:GetBool (info)
  return self.db.profile[info[#info]]
end
function TwinCam:SetBool (info, v)
  self.db.profile[info[#info]] = v
end
function TwinCam:GetColor (info)
  local c = self.db.profile[info[#info]]
  return c.r, c.g, c.b, (c.a or 1)
end
function TwinCam:SetColor (info, r, g, b, a)
  local c = { r = r, g = g, b = b, a = a }
  self.db.profile[info[#info]] = c
end

-- Helpers for debugging
function TwinCam:debug_sf ()
  TwinCam:COMBAT_LOG_EVENT_UNFILTERED ("", 0, "SPELL_AURA_APPLIED",0x0000000000000000,nil,0x80000000,0x000000000194E5F2,UnitName("player"),0x514,45270,"Shadowfury",0x20,"DEBUFF")
end

function TwinCam:debug_emote()
  TwinCam:CHAT_MSG_RAID_BOSS_EMOTE("CHAT_MSG_RAID_BOSS_EMOTE", "Alythess directs Conflagration at Someone", GWA, "en", "whatever", self.name)
end


-- Detect Conflagration via the boss emote
function TwinCam:CHAT_MSG_RAID_BOSS_EMOTE(eventName, msg, sender, lang, channel, target)
  if ((sender == GWA) or (sender == LS)) and (target == self.name) and
     (msg:match (".*Conflagration.*"))
  then
    self:OnConflag ();
  end
end

function TwinCam:COMBAT_LOG_EVENT_UNFILTERED(eventName, time, event, srcId, srcName, srcFlags, dstID, dstName, dstFlags, spellID, spellName)
  -- Only care about shadow fury affecting the player
  if (dstName == self.name) and (spellName == "Shadowfury") then
    if (event == "SPELL_AURA_APPLIED") then
      self:OnShadowfury ()
    elseif (event == "SPELL_AURA_REMOVED") then
      self:StopFlashShadowfury ()
    end
  end
end

function TwinCam:PLAYER_DEAD(eventName)
  self:StopFlashShadowfury ()
end

function TwinCam:OnConflag ()
  if self.db.profile.conflagSound then self:PlayRunAway () end
  if self.db.profile.conflagVisible then self:FlashConflag () end
end

function TwinCam:PlayRunAway ()
  PlaySoundFile (RunAwaySound)
end

function TwinCam:FlashConflag ()
  local c = self.db.profile.conflagColor
  self.conflagtex:SetVertexColor (c.r, c.g, c.b)
  UIFrameFlash (self.conflagtex, 0, 0.5, 5.1, false, 0, 0)
end

function TwinCam:OnShadowfury ()
  if self.db.profile.shadowfuryVisible then self:FlashShadowfury () end
end

function TwinCam:FlashShadowfury ()
  local c = self.db.profile.shadowfuryColor
  self.sftex:SetVertexColor (c.r, c.g, c.b)
  self.sftex:SetAlpha (1)
end

function TwinCam:StopFlashShadowfury ()
  self.sftex:SetAlpha (0)
end

function TwinCam:OnInitialize ()
  self.name = UnitName ("player")

  -- Set up configuration panel
  LibStub ("AceConfigRegistry-3.0"):RegisterOptionsTable (NAME, options)
  local ACD = LibStub ("AceConfigDialog-3.0")
  ACD:AddToBlizOptions (NAME, GetAddOnMetadata (ADDON_NAME, "Title"))
  self:RegisterChatCommand ("twincam", function()
    InterfaceOptionsFrame_OpenToFrame(ACD.BlizOptions[NAME].frame)
  end)
  self:RegisterChatCommand ("twincambonus", function() PlaySoundFile(LamentSound) end)

  -- Prepare config db
  self.db = LibStub ("AceDB-3.0"):New ("TwinCamDB", defaults)
  options.args.profiles = LibStub ("AceDBOptions-3.0"):GetOptionsTable (self.db)

  -- Our overlay frame
  local overlayframe = CreateFrame ("Frame", nil, WorldFrame)
  overlayframe:SetClampedToScreen (true)
  overlayframe:SetFrameStrata ("BACKGROUND")
  overlayframe:SetAllPoints (UIParent)
  overlayframe:Show ()
  self.overlayFrame = overlayframe

  -- Conflag 'skull' texture (from the raid marker)
  local conflagtex = overlayframe:CreateTexture ("TwinCamConflagTexture")
  conflagtex:SetAlpha (0)
  conflagtex:SetBlendMode ("ADD")
  conflagtex:SetAllPoints (overlayframe)
  conflagtex:SetTexture (TargetRaidTargetIcon:GetTexture ())
  -- 1 star, 2 circle, 3 diamond, 4 triangle, 5 moon, 6 square, 7 cross, 8 skull
  SetRaidTargetIconTexture (conflagtex, 8)
  self.conflagtex = conflagtex

  -- Shadowfury texture (from tooltip background)
  local sftex = overlayframe:CreateTexture ("TwinCamShadowfuryTexture")
  sftex:SetAlpha (0)
  sftex:SetBlendMode ("DISABLE")
  sftex:SetAllPoints (overlayframe)
  sftex:SetTexture ("Interface/Tooltips/UI-Tooltip-Background")
  self.sftex = sftex
end

function TwinCam:OnEnable ()
  self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
  self:RegisterEvent("CHAT_MSG_RAID_BOSS_EMOTE")
  self:RegisterEvent("PLAYER_DEAD")
end

